#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/19 10:25:51

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
import random
# ===============================================================================
'''
'''
# ===============================================================================

__all__ = [
            'pr',
            'init_seeds',
           ]

def pr(data, info = None):
    '''print detail data information
    '''
    if info is not None:
        msg = f"==== {info} ==== "
    else:
        msg = ''
    msg += f'type:{type(data)} id:{id(data)}\n'

    if isinstance(data, np.ndarray):
        msg += f'    shape:{data.shape}\n' \
               f'    dtype:{data.dtype}\n'
    elif isinstance(data, np.dtype):
        print()
    msg += f'{data}'
    print(msg)


def init_seeds(seed=0):
    # Initialize random number generator (RNG) seeds
    random.seed(seed)
    np.random.seed(seed)


#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/18 22:59:51

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from functools import partial
# ===============================================================================
''' 包装了 sklearn 所支持的数据集
'''
# ===============================================================================

dataset_list = [
    "make_biclusters",
    "make_blobs",
    "make_circles",
    "make_classification",
    "make_checkerboard",
    "make_friedman1",
    "make_friedman2",
    "make_friedman3",
    "make_gaussian_quantiles",
    "make_hastie_10_2",
    "make_low_rank_matrix",
    "make_moons",
    "make_multilabel_classification",
    "make_regression",
    "make_s_curve",
    "make_sparse_coded_signal",
    "make_sparse_spd_matrix",
    "make_sparse_uncorrelated",
    "make_spd_matrix",
    "make_swiss_roll",

    "load_boston",
    "load_diabetes",
    "load_digits",
    # "load_files",
    "load_iris",
    "load_breast_cancer",
    "load_linnerud",
    # "load_sample_image",
    "load_sample_images",
    # "load_svmlight_file",
    # "load_svmlight_files",
    "load_wine",

    "fetch_20newsgroups",
    "fetch_20newsgroups_vectorized",
    "fetch_lfw_pairs",
    "fetch_lfw_people",
    "fetch_olivetti_faces",
    "fetch_species_distributions",
    "fetch_california_housing",
    "fetch_covtype",
    "fetch_rcv1",
    "fetch_kddcup99",
    "fetch_openml",
]

def get_dataset(_name = 'make_blobs', **kws):
    name = _name.lower()
    ##### make
    if name == 'make_biclusters':
        from sklearn.datasets import make_biclusters
        dataset = partial(make_biclusters, shape = (2,2), n_clusters=4)
    elif name == 'make_blobs':
        from sklearn.datasets import make_blobs as dataset
    elif name == 'make_circles':
        from sklearn.datasets import make_circles as dataset
    elif name == 'make_classification':
        from sklearn.datasets import make_classification as dataset
    elif name == 'make_checkerboard':
        from sklearn.datasets import make_checkerboard
        dataset = partial(make_checkerboard, shape=(2, 2), n_clusters=4)
    elif name == 'make_friedman1':
        from sklearn.datasets import make_friedman1 as dataset
    elif name == 'make_friedman2':
        from sklearn.datasets import make_friedman2 as dataset
    elif name == 'make_friedman3':
        from sklearn.datasets import make_friedman3 as dataset
    elif name == 'make_gaussian_quantiles':
        from sklearn.datasets import make_gaussian_quantiles as dataset
    elif name == 'make_hastie_10_2':
        from sklearn.datasets import make_hastie_10_2 as dataset
    elif name == 'make_low_rank_matrix':
        from sklearn.datasets import make_low_rank_matrix as dataset
    elif name == 'make_moons':
        from sklearn.datasets import make_moons as dataset
    elif name == 'make_multilabel_classification':
        from sklearn.datasets import make_multilabel_classification as dataset
    elif name == 'make_regression':
        from sklearn.datasets import make_regression as dataset
    elif name == 'make_s_curve':
        from sklearn.datasets import make_s_curve as dataset
    elif name == 'make_sparse_coded_signal':
        from sklearn.datasets import make_sparse_coded_signal
        dataset = partial(make_sparse_coded_signal, n_samples = 100, n_components = 3, n_features = 2, n_nonzero_coefs = 0)
    elif name == 'make_sparse_spd_matrix':
        from sklearn.datasets import make_sparse_spd_matrix
        dataset = partial(make_sparse_spd_matrix, dim=1)
    elif name == 'make_sparse_uncorrelated':
        from sklearn.datasets import make_sparse_uncorrelated as dataset
    elif name == 'make_spd_matrix':
        from sklearn.datasets import make_spd_matrix
        dataset = partial(make_spd_matrix, n_dim=1)
    elif name == 'make_swiss_roll':
        from sklearn.datasets import make_swiss_roll as dataset

    ##### load
    elif name == 'load_boston':
        from sklearn.datasets import load_boston as dataset
    elif name == 'load_diabetes':
        from sklearn.datasets import load_diabetes as dataset
    elif name == 'load_digits':
        from sklearn.datasets import load_digits as dataset
    elif name == 'load_iris':
        from sklearn.datasets import load_iris as dataset
    elif name == 'load_breast_cancer':
        from sklearn.datasets import load_breast_cancer as dataset
    elif name == 'load_linnerud':
        from sklearn.datasets import load_linnerud as dataset
    elif name == 'load_sample_images':
        from sklearn.datasets import load_sample_images as dataset
    elif name == 'load_wine':
        from sklearn.datasets import load_wine as dataset

    ##### fetch
    elif name == 'fetch_20newsgroups':
        from sklearn.datasets import fetch_20newsgroups as dataset
    elif name == 'fetch_20newsgroups_vectorized':
        from sklearn.datasets import fetch_20newsgroups_vectorized as dataset
    elif name == 'fetch_lfw_pairs':
        from sklearn.datasets import fetch_lfw_pairs as dataset
    elif name == 'fetch_lfw_people':
        from sklearn.datasets import fetch_lfw_people as dataset
    elif name == 'fetch_olivetti_faces':
        from sklearn.datasets import fetch_olivetti_faces as dataset
    elif name == 'fetch_species_distributions':
        from sklearn.datasets import fetch_species_distributions as dataset
    elif name == 'fetch_california_housing':
        from sklearn.datasets import fetch_california_housing as dataset
    elif name == 'fetch_covtype':
        from sklearn.datasets import fetch_covtype as dataset
    elif name == 'fetch_rcv1':
        from sklearn.datasets import fetch_rcv1 as dataset
    elif name == 'fetch_kddcup99':
        from sklearn.datasets import fetch_kddcup99 as dataset
    elif name == 'fetch_openml':
        from sklearn.datasets import fetch_openml as dataset
    ##### 其他加载器
    elif name == 'load_files':
        from sklearn.datasets import load_files as dataset
    elif name == 'load_sample_image':
        from sklearn.datasets import load_sample_image as dataset
    elif name == 'load_svmlight_file':
        from sklearn.datasets import load_svmlight_file as dataset
    elif name == 'load_svmlight_files':
        from sklearn.datasets import load_svmlight_files as dataset
    else:
        raise RuntimeError("Cann't create {}".format(_name))

    return dataset(**kws)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/5/11 21:54:21

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
#===============================================================================
'''  包装了 sklearn所支持的聚类算法
'''
#===============================================================================

cluster_list = [
    'AffinityPropagation',
    'Agglomerative',
    'Birch',
    'DBScan',
    'DBScan_daal'
    'GaussianMixture',
    'BayesianGaussianMixture',
    'KMeans',
    'KMeans_daal',
    'KMeansBisecting',
    'KMeansMiniBatch',
    'MeanShift',
    'OPTICS',
    'Spectral',
    'SpectralBiing',
    'SpectralCoing'
]

def get_cluster(_name = "KMeans", **kws):
    name = _name.lower()
    if name == 'AffinityPropagation'.lower():
        from sklearn.cluster import AffinityPropagation as Alg
    elif name == 'Agglomerative'.lower():
        from sklearn.cluster import AgglomerativeClustering as Alg
    elif name == 'Birch'.lower():
        from sklearn.cluster import Birch as Alg
    elif name == 'DBScan'.lower():
        from sklearn.cluster import DBSCAN as Alg
    elif name == 'DBScan_daal'.lower(): 
        from daal4py.sklearn.cluster import DBSCAN as Alg
    elif name == 'GaussianMixture'.lower(): 
        from sklearn.mixture import GaussianMixture as Alg
    elif name == 'BayesianGaussianMixture'.lower(): 
        from sklearn.mixture import BayesianGaussianMixture as Alg
    elif name == 'KMeans'.lower(): 
        from sklearn.cluster import KMeans as Alg
    elif name == 'KMeans_daal'.lower(): 
        from daal4py.sklearn.cluster import KMeans as Alg
    elif name == 'KMeansBisecting'.lower(): 
        from sklearn.cluster import BisectingKMeans as Alg
    elif name == 'KMeansMiniBatch'.lower(): 
        from sklearn.cluster import MiniBatchKMeans as Alg
    elif name == 'MeanShift'.lower(): 
        from sklearn.cluster import MeanShift as Alg
    elif name == 'OPTICS'.lower(): 
        from sklearn.cluster import OPTICS as Alg
    elif name == 'Spectral'.lower(): 
        from sklearn.cluster import SpectralClustering as Alg
    elif name == 'SpectralBiing'.lower(): 
        from sklearn.cluster import SpectralBiclustering as Alg
    elif name == 'SpectralCoing'.lower(): 
        from sklearn.cluster import SpectralCoclustering as Alg
    else:
        raise RuntimeError("Cann't create {}".format(_name))

    return Alg(**kws)





#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022.08

@author: laok
@copyright: Apache License, Version 2.0
'''
from collections import OrderedDict
# ===============================================================================
#                                                                                
# ===============================================================================
__all__ = [ 'file_read_text', 'file_read_bin', 
            'file_write_text', 'file_write_bin',
            'file_read_lines', 'file_write_lines',
            'ClsIdx']

def file_read_text(fname, encoding='utf8'):
    with open(fname, mode='r', encoding=encoding) as f:
        return f.read()

def file_write_text(fname, text, encoding='utf8'):
    with open(fname, mode='w', encoding=encoding) as f:
        f.write(text)

def file_read_lines(fname, encoding='utf8', strip = True):
    with open(fname, mode='r', encoding=encoding) as f:
        line = f.readline()
        if strip:
            line = line.strip()
        yield line

def file_write_lines(fname, lines, encoding='utf8', line_append='\n'):
    with open(fname, mode='w', encoding=encoding) as f:
        for line in lines:
            f.write(line + line_append)

def file_read_bin(fname):
    with open(fname, mode='rb') as f:
        return f.read()

def file_write_bin(fname, data):
    with open(fname, mode='wb') as f:
        f.write(data)


class ClsIdx:
    def __init__(self, cls_file = None):
        self.cls_data_ = OrderedDict()
        if cls_file:
            self.setClsFile(cls_file)

    def setClsFile(self, cls_file):
        with open(cls_file) as f:
            for i,line in enumerate(f):
                self.cls_data_[i] = line.strip()
        return self

    def getName(self, idx):
        _idx = int(idx)
        return self.cls_data_.get(_idx, '')

    def getIndex(self, name):
        if name == None or name == '':
            return -1
        for _idx, _name in self.cls_data_.items():
            if name in _name:
                return _idx
        return -1

    def show(self):
        for k,v in self.cls_data_.items():
            print(f'{k}={v}')



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/25 20:12:59

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''

# ===============================================================================
'''
'''
# ===============================================================================

__all__ = ['show_plt']

def show_plt(img, cmap=None, norm=None, aspect=None, interpolation=None,
        alpha=None, vmin=None, vmax=None, origin=None, extent=None, *,
        interpolation_stage=None, filternorm=True, filterrad=4.0,
        resample=None, url=None, data=None, **kwargs):
    import matplotlib.pyplot as plt
    plt.imshow(img,cmap=cmap, norm=norm, aspect=aspect, interpolation=interpolation,
        alpha=alpha, vmin=vmin, vmax=vmax, origin=origin, extent=extent,
        interpolation_stage=interpolation_stage, filternorm=filternorm, filterrad=filterrad,
        resample=resample, url=url, data=data, **kwargs)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2020/6/4 18:13:09

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import inspect, contextlib, types
#===============================================================================
# 
#===============================================================================

__all__ = ['dump']

def dump_help(obj):
    with open('help-%s.txt' % _get_name(obj), 'w') as f, contextlib.redirect_stdout(f):
        help(obj)

# _obj = object()

def dump(obj, ignore_fields = tuple()):

    # 空对象,直接不处理
    if obj is None:
        print('None')
        return

    # allFunc = []
    # allAttr = []
    # allClass = []
    # allModules = []
    # allWMethWrapper = []
    # allMethod = []

    all_Attr = []
    all_Class = []
    all_ModuleType = []
    all_MethodType = []
    all_FunctionType = []
    all_WrapperDescriptorType = []
    all_MethodWrapperType = []
    all_MethodDescriptorType = []
    all_BuiltinFunctionType = []
    all_BuiltinMethodType = []

    print('\n# -----', type(obj))
    for item in _get_attr_pair(obj):

        k, v = item

        if k in ignore_fields: # 忽略的字段
            continue

        if k.startswith('_') : # 私有变量不访问
            if not k.startswith('__') or not k.endswith('__'):
                continue

        if isinstance(v, type):
            all_Class.append(item)
        elif isinstance(v, types.ModuleType):
            all_ModuleType.append(item)
        elif isinstance(v, types.FunctionType):
            all_FunctionType.append(item)
        elif isinstance(v, types.MethodType):
            all_MethodType.append(item)
        elif isinstance(v, types.WrapperDescriptorType):
            all_WrapperDescriptorType.append(item)
        elif isinstance(v, types.MethodWrapperType):
            all_MethodWrapperType.append(item)
        elif isinstance(v, types.MethodDescriptorType):
            all_MethodDescriptorType.append(item)
        elif isinstance(v, types.BuiltinFunctionType):
            all_BuiltinFunctionType.append(item)
        elif isinstance(v, types.BuiltinMethodType):
            all_BuiltinMethodType.append(item)
        else:
            all_Attr.append(item)

    if all_ModuleType:
        print('# module ---->')
        for k,v in all_ModuleType:
            print('\t# %s [%s]' %(k, _get_doc(v)) )

    if all_Class:
        print('# class ---->')
        for k,v in all_Class:
            print('\t# %s [%s]' %(k, _get_doc(v)) )

    if all_Attr:
        print('# attr ---->')
        for k,v in all_Attr:
            print('\t# %s[%s] [%s]' %(k, str(v).replace('\n', ' ')[0:100], _get_doc(v)) )

    if any([all_FunctionType, all_MethodType, all_WrapperDescriptorType, all_MethodWrapperType,
            all_MethodDescriptorType,  all_BuiltinFunctionType, all_BuiltinMethodType]):
        print('# func ---->')

    if all_FunctionType:
        # print('# func ---->')
        for k,v in all_FunctionType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_MethodType:
        # print('# method ---->')
        for k,v in all_MethodType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_WrapperDescriptorType:
        # print('# warpper descriptor ---->')
        for k,v in all_WrapperDescriptorType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_MethodWrapperType:
        # print('# method warpper ---->')
        for k,v in all_MethodWrapperType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_MethodDescriptorType:
        # print('# method descriptor ---->')
        for k,v in all_MethodDescriptorType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_BuiltinFunctionType:
        # print('# builtin func ---->')
        for k,v in all_BuiltinFunctionType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    if all_BuiltinMethodType:
        # print('# builtin method ---->')
        for k,v in all_BuiltinMethodType:
            print('\t# %s%s [%s]' %(k, _get_signature(v), _get_doc(v)) )

    print()

def _get_doc(v):
    if isinstance(v, str):
        return  '`str`'
    if isinstance(v, bytes):
        return  '`bytes`'
    elif isinstance(v, int):
        return  '`integer`'
    elif isinstance(v, dict):
        return  '`dict`'
    elif isinstance(v, set):
        return  '`set`'
    elif isinstance(v, frozenset):
        return '`fronzenset`'
    elif isinstance(v, float):
        return '`float`'
    elif isinstance(v, tuple):
        return '`tuple`'
    elif isinstance(v, list):
        return '`list`'

    try:
        doc = inspect.getdoc(v)
        return doc.replace('\n', ' ')[0:250]
    except Exception as e:
        return  ''

def _get_signature(v):
    try:
        return str(inspect.signature(v))
    except: # 有些函数没有签名
        return "(...)"

def _get_name(v):
    try:
        name = v.__name__
    except Exception:
        try:
            name = v.__class__.__name__
        except Exception:
            name = 'none'
    return name

def _get_attr_pair(obj):
    '''
    获取可能的属性列表
    '''

    # 默认从 __all__获取属性
    datalist = []
    for k in getattr(obj, '__all__', tuple()):
        if hasattr(obj, k):
            datalist.append( (k, getattr(obj, k)) )

    # 获取所有属性
    if not datalist:
        datalist = inspect.getmembers(obj)

    datalist.sort(key=lambda kv: kv[0])

    # 把 __init__放在最前面
    for i,(k,v) in enumerate(datalist):
        if k == '__init__':
            del datalist[i]
            datalist.insert(0, (k, v))
            break

    return datalist



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/19 11:11:45

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
# ===============================================================================
'''
'''
# ===============================================================================

classify_list = [
    "alexnet",
    "googlenet",
    "inception_v3",
    "vgg11", "vgg11_bn", "vgg13", "vgg13_bn", "vgg16", "vgg16_bn", "vgg19", "vgg19_bn",
    "convnext_base", "convnext_large", "convnext_small", "convnext_tiny",
    "densenet121", "densenet161","densenet169", "densenet201",
    "efficientnet_b0", "efficientnet_b1", "efficientnet_b2", "efficientnet_b3", "efficientnet_b4", "efficientnet_b5", "efficientnet_b6","efficientnet_b7",
    "efficientnet_v2_l", "efficientnet_v2_m", "efficientnet_v2_s",
    "mnasnet0_5", "mnasnet0_75", "mnasnet1_0", "mnasnet1_3",
    "mobilenet_v2", "mobilenet_v3_large", "mobilenet_v3_small",
    "regnet_x_16gf", "regnet_x_1_6gf", "regnet_x_32gf", "regnet_x_3_2gf", "regnet_x_400mf", "regnet_x_800mf", "regnet_x_8gf",
    "regnet_y_128gf", "regnet_y_16gf", "regnet_y_1_6gf", "regnet_y_32gf", "regnet_y_3_2gf", "regnet_y_400mf", "regnet_y_800mf", "regnet_y_8gf",
    "resnet18", "resnet34", "resnet50", "resnet101", "resnet152", "resnext101_32x8d", "resnext101_64x4d", "resnext50_32x4d", "wide_resnet101_2", "wide_resnet50_2",
    "shufflenet_v2_x0_5", "shufflenet_v2_x1_0", "shufflenet_v2_x1_5", "shufflenet_v2_x2_0",
    "squeezenet1_0", "squeezenet1_1",
    "swin_b", "swin_s", "swin_t",
    "vit_b_16", "vit_b_32", "vit_h_14", "vit_l_16", "vit_l_32",
]

def get_classify(name, **kws):
    name = name.lower()
    if name == "alexnet":
        from torchvision.models import alexnet as Alg
    elif name == "convnext_base":
        from torchvision.models import convnext_base as Alg
    elif name == "convnext_large":
        from torchvision.models import convnext_large as Alg
    elif name == "convnext_small":
        from torchvision.models import convnext_small as Alg
    elif name == "convnext_tiny":
        from torchvision.models import convnext_tiny as Alg
    elif name == "densenet121":
        from torchvision.models import densenet121 as Alg
    elif name == "densenet161":
        from torchvision.models import densenet161 as Alg
    elif name == "densenet169":
        from torchvision.models import densenet169 as Alg
    elif name == "densenet201":
        from torchvision.models import densenet201 as Alg
    elif name == "efficientnet_b0":
        from torchvision.models import efficientnet_b0 as Alg
    elif name == "efficientnet_b1":
        from torchvision.models import efficientnet_b1 as Alg
    elif name == "efficientnet_b2":
        from torchvision.models import efficientnet_b2 as Alg
    elif name == "efficientnet_b3":
        from torchvision.models import efficientnet_b3 as Alg
    elif name == "efficientnet_b4":
        from torchvision.models import efficientnet_b4 as Alg
    elif name == "efficientnet_b5":
        from torchvision.models import efficientnet_b5 as Alg
    elif name == "efficientnet_b6":
        from torchvision.models import efficientnet_b6 as Alg
    elif name == "efficientnet_b7":
        from torchvision.models import efficientnet_b7 as Alg
    elif name == "efficientnet_v2_l":
        from torchvision.models import efficientnet_v2_l as Alg
    elif name == "efficientnet_v2_m":
        from torchvision.models import efficientnet_v2_m as Alg
    elif name == "efficientnet_v2_s":
        from torchvision.models import efficientnet_v2_s as Alg
    elif name == "googlenet":
        from torchvision.models import googlenet as Alg
    elif name == "inception_v3":
        from torchvision.models import inception_v3 as Alg
    elif name == "mnasnet0_5":
        from torchvision.models import mnasnet0_5 as Alg
    elif name == "mnasnet0_75":
        from torchvision.models import mnasnet0_75 as Alg
    elif name == "mnasnet1_0":
        from torchvision.models import mnasnet1_0 as Alg
    elif name == "mnasnet1_3":
        from torchvision.models import mnasnet1_3 as Alg
    elif name == "mobilenet_v2":
        from torchvision.models import mobilenet_v2 as Alg
    elif name == "mobilenet_v3_large":
        from torchvision.models import mobilenet_v3_large as Alg
    elif name == "mobilenet_v3_small":
        from torchvision.models import mobilenet_v3_small as Alg
    elif name == "regnet_x_16gf":
        from torchvision.models import regnet_x_16gf as Alg
    elif name == "regnet_x_1_6gf":
        from torchvision.models import regnet_x_1_6gf as Alg
    elif name == "regnet_x_32gf":
        from torchvision.models import regnet_x_32gf as Alg
    elif name == "regnet_x_3_2gf":
        from torchvision.models import regnet_x_3_2gf as Alg
    elif name == "regnet_x_400mf":
        from torchvision.models import regnet_x_400mf as Alg
    elif name == "regnet_x_800mf":
        from torchvision.models import regnet_x_800mf as Alg
    elif name == "regnet_x_8gf":
        from torchvision.models import regnet_x_8gf as Alg
    elif name == "regnet_y_128gf":
        from torchvision.models import regnet_y_128gf as Alg
    elif name == "regnet_y_16gf":
        from torchvision.models import regnet_y_16gf as Alg
    elif name == "regnet_y_1_6gf":
        from torchvision.models import regnet_y_1_6gf as Alg
    elif name == "regnet_y_32gf":
        from torchvision.models import regnet_y_32gf as Alg
    elif name == "regnet_y_3_2gf":
        from torchvision.models import regnet_y_3_2gf as Alg
    elif name == "regnet_y_400mf":
        from torchvision.models import regnet_y_400mf as Alg
    elif name == "regnet_y_800mf":
        from torchvision.models import regnet_y_800mf as Alg
    elif name == "regnet_y_8gf":
        from torchvision.models import regnet_y_8gf as Alg
    elif name == "resnet101":
        from torchvision.models import resnet101 as Alg
    elif name == "resnet152":
        from torchvision.models import resnet152 as Alg
    elif name == "resnet18":
        from torchvision.models import resnet18 as Alg
    elif name == "resnet34":
        from torchvision.models import resnet34 as Alg
    elif name == "resnet50":
        from torchvision.models import resnet50 as Alg
    elif name == "resnext101_32x8d":
        from torchvision.models import resnext101_32x8d as Alg
    elif name == "resnext101_64x4d":
        from torchvision.models import resnext101_64x4d as Alg
    elif name == "resnext50_32x4d":
        from torchvision.models import resnext50_32x4d as Alg
    elif name == "shufflenet_v2_x0_5":
        from torchvision.models import shufflenet_v2_x0_5 as Alg
    elif name == "shufflenet_v2_x1_0":
        from torchvision.models import shufflenet_v2_x1_0 as Alg
    elif name == "shufflenet_v2_x1_5":
        from torchvision.models import shufflenet_v2_x1_5 as Alg
    elif name == "shufflenet_v2_x2_0":
        from torchvision.models import shufflenet_v2_x2_0 as Alg
    elif name == "squeezenet1_0":
        from torchvision.models import squeezenet1_0 as Alg
    elif name == "squeezenet1_1":
        from torchvision.models import squeezenet1_1 as Alg
    elif name == "swin_b":
        from torchvision.models import swin_b as Alg
    elif name == "swin_s":
        from torchvision.models import swin_s as Alg
    elif name == "swin_t":
        from torchvision.models import swin_t as Alg
    elif name == "vgg11":
        from torchvision.models import vgg11 as Alg
    elif name == "vgg11_bn":
        from torchvision.models import vgg11_bn as Alg
    elif name == "vgg13":
        from torchvision.models import vgg13 as Alg
    elif name == "vgg13_bn":
        from torchvision.models import vgg13_bn as Alg
    elif name == "vgg16":
        from torchvision.models import vgg16 as Alg
    elif name == "vgg16_bn":
        from torchvision.models import vgg16_bn as Alg
    elif name == "vgg19":
        from torchvision.models import vgg19 as Alg
    elif name == "vgg19_bn":
        from torchvision.models import vgg19_bn as Alg
    elif name == "vit_b_16":
        from torchvision.models import vit_b_16 as Alg
    elif name == "vit_b_32":
        from torchvision.models import vit_b_32 as Alg
    elif name == "vit_h_14":
        from torchvision.models import vit_h_14 as Alg
    elif name == "vit_l_16":
        from torchvision.models import vit_l_16 as Alg
    elif name == "vit_l_32":
        from torchvision.models import vit_l_32 as Alg
    elif name == "wide_resnet101_2":
        from torchvision.models import wide_resnet101_2 as Alg
    elif name == "wide_resnet50_2":
        from torchvision.models import wide_resnet50_2 as Alg
    return Alg



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/5/23 01:30:30

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''

#===============================================================================
'''     
'''
#===============================================================================
classifier_list = [
    ##### 线性
    'RidgeClassifier',
    'LogisticRegression', 'LogisticRegressionCV',
    'SGDClassifier',
    'Perceptron',
    'PassiveAggressiveClassifier',
    'SGDOneClassSVM',

    ##### 判别分析
    'LinearDiscriminantAnalysis', 'QuadraticDiscriminantAnalysis',

    ##### SVM
    'SVC', 'NuSVC', 'LinearSVC', 'OneClassSVM',

    ##### KNN
    'KNeighborsClassifier', 'RadiusNeighborsClassifier',
    'NearestCentroid',

    ##### 高斯处理
    'GaussianProcessClassifier',

    ##### 贝叶斯
    "BernoulliNB", "GaussianNB", "MultinomialNB", "ComplementNB", "CategoricalNB",

    ##### 决策树
    'DecisionTreeClassifier', 'ExtraTreeClassifier',

    ##### 集成方法
    "RandomForestClassifier", "ExtraTreesClassifier", "BaggingClassifier",
    "IsolationForest", "GradientBoostingClassifier", "AdaBoostClassifier",
    "VotingClassifier", "StackingClassifier", "HistGradientBoostingClassifier",

    ##### 神经网络
    'MLPClassifier','',
]

def get_classifier(_name = "SVC", **kws):
    name = _name.lower()
    if name == 'RidgeClassifier'.lower():
        from sklearn.linear_model import RidgeClassifier as Alg
    elif name == 'LogisticRegression'.lower():
        from sklearn.linear_model import LogisticRegression as Alg
    elif name == 'LogisticRegressionCV'.lower():
        from sklearn.linear_model import LogisticRegressionCV as Alg
    elif name == 'SGDClassifier'.lower():
        from sklearn.linear_model import SGDClassifier as Alg
    elif name == 'Perceptron'.lower():
        from sklearn.linear_model import Perceptron as Alg
    elif name == 'PassiveAggressiveClassifier'.lower():
        from sklearn.linear_model import PassiveAggressiveClassifier as Alg
    elif name == 'SGDOneClassSVM'.lower():
        from sklearn.linear_model import SGDOneClassSVM as Alg

    elif name == 'LinearDiscriminantAnalysis'.lower():
        from sklearn.discriminant_analysis import LinearDiscriminantAnalysis as Alg
    elif name == 'QuadraticDiscriminantAnalysis'.lower():
        from sklearn.discriminant_analysis import QuadraticDiscriminantAnalysis as Alg

    elif name == 'SVC'.lower():
        from sklearn.svm import SVC as Alg
    elif name == 'NuSVC'.lower():
        from sklearn.svm import NuSVC as Alg
    elif name == 'LinearSVC'.lower():
        from sklearn.svm import LinearSVC as Alg
    elif name == 'OneClassSVM'.lower():
        from sklearn.svm import OneClassSVM as Alg

    elif name == 'KNeighborsClassifier'.lower():
        from sklearn.neighbors import KNeighborsClassifier as Alg
    elif name == 'RadiusNeighborsClassifier'.lower():
        from sklearn.neighbors import RadiusNeighborsClassifier as Alg
    elif name == 'NearestCentroid'.lower():
        from sklearn.neighbors import NearestCentroid as Alg

    elif name == 'GaussianProcessClassifier'.lower():
        from sklearn.gaussian_process import GaussianProcessClassifier as Alg

    elif name == 'BernoulliNB'.lower():
        from sklearn.naive_bayes import BernoulliNB as Alg
    elif name == 'GaussianNB'.lower():
        from sklearn.naive_bayes import GaussianNB as Alg
    elif name == 'MultinomialNB'.lower():
        from sklearn.naive_bayes import MultinomialNB as Alg
    elif name == 'ComplementNB'.lower():
        from sklearn.naive_bayes import ComplementNB as Alg
    elif name == 'CategoricalNB'.lower():
        from sklearn.naive_bayes import CategoricalNB as Alg

    elif name == 'DecisionTreeClassifier'.lower():
        from sklearn.tree import DecisionTreeClassifier as Alg    
    elif name == 'ExtraTreeClassifier'.lower():
        from sklearn.tree import ExtraTreeClassifier as Alg    

    elif name == 'RandomForestClassifier'.lower():
        from sklearn.tree import RandomForestClassifier as Alg    
    elif name == 'ExtraTreesClassifier'.lower():
        from sklearn.tree import ExtraTreesClassifier as Alg    
    elif name == 'BaggingClassifier'.lower():
        from sklearn.tree import BaggingClassifier as Alg    
    elif name == 'IsolationForest'.lower():
        from sklearn.tree import IsolationForest as Alg    
    elif name == 'GradientBoostingClassifier'.lower():
        from sklearn.tree import GradientBoostingClassifier as Alg    
    elif name == 'AdaBoostClassifier'.lower():
        from sklearn.tree import AdaBoostClassifier as Alg    
    elif name == 'VotingClassifier'.lower():
        from sklearn.tree import VotingClassifier as Alg    
    elif name == 'StackingClassifier'.lower():
        from sklearn.tree import StackingClassifier as Alg           
    elif name == 'HistGradientBoostingClassifier'.lower():
        from sklearn.tree import HistGradientBoostingClassifier as Alg       

    elif name == 'MLPClassifier'.lower():
        from sklearn.neural_network import MLPClassifier as Alg     
    else:
        raise RuntimeError("Cann't create {}".format(_name))

    return Alg(**kws)





#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/8/10 12:06:24

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import rosbag
# ===============================================================================
'''遍历/读取 rosbag数据
'''
# ===============================================================================

__all__ = ['rosbag_iter', 'rosbag_iter_img', 'rosbag_iter_pcd', 'save_pcd', 'save_img']

def rosbag_iter(rosbag_name, topic_name = None):
    with rosbag.Bag(rosbag_name, "r") as bag:  # 要读取的bag文件；
        for topic, msg, t in bag.read_messages():
            if topic_name is None:
                yield topic, msg, t
            elif topic_name == topic:
                yield topic, msg, t

def rosbag_iter_img(rosbag_name, topic_name = None):
    from cv_bridge import CvBridge
    bridge = CvBridge()
    for topic, msg, t in rosbag_iter(rosbag_name, topic_name):
        cv_image = bridge.imgmsg_to_cv2(msg, "bgr8")
        yield cv_image, msg.header.stamp.to_sec()

def rosbag_iter_pcd(rosbag_name, topic_name = None):
    import sensor_msgs.point_cloud2 as pc2
    import numpy as np

    for topic, msg, t in rosbag_iter(rosbag_name, topic_name):
        points_data = np.array(list(pc2.read_points(msg)))
        lidar = list(np.delete(points_data, np.where(np.isnan(points_data))[0], axis=0))
        yield lidar, msg.header.stamp.to_sec()

def save_pcd(lidar, filename, binary=True):
    if binary:
        _save_pcd_binary(lidar, filename)
    else:
        _save_pcd_ascii(lidar, filename)

def _save_pcd_binary(lidar, filename):
    PCD_BINARY_TEMPLATE = """VERSION 0.7
FIELDS x y z intensity
SIZE 4 4 4 4
TYPE F F F F
COUNT 1 1 1 1
WIDTH {}
HEIGHT 1
VIEWPOINT 0 0 0 1 0 0 0
POINTS {}
DATA binary
"""

    import struct
    sz = len(lidar)
    with open(filename, 'wb') as f:
        header = PCD_BINARY_TEMPLATE.format(sz, sz)
        f.write(header.encode())
        for pi in lidar:
            h = struct.pack('ffff', pi[0], pi[1], pi[2], pi[3])
            f.write(h)

def _save_pcd_ascii(lidar, filename):
    PCD_ASCII_TEMPLATE = """VERSION 0.7
FIELDS x y z intensity
SIZE 4 4 4 2
TYPE F F F U
COUNT 1 1 1 1
WIDTH {}
HEIGHT 1
VIEWPOINT 0 0 0 1 0 0 0
POINTS {}
DATA ascii
"""
    sz = len(lidar)
    with open(filename, 'w') as f:
        header = PCD_ASCII_TEMPLATE.format(sz, sz)
        f.write(header)
        for pi in lidar:
            f.write(' '.join([str(i) for i in pi]) + '\n')

def save_img(img, filename):
    import os, cv2
    name, ext = os.path.splitext(filename)
    retval, buf = cv2.imencode(ext, img)
    return buf.tofile(filename)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2020/6/4 14:41:10

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
from ._dump import *
from ._run import *
from pprint import pprint; pprint
import sys as _sys
import types as _types
#===============================================================================
#
#===============================================================================

try:
    main_module = _sys.modules['__main__']
except:
    pass

def print_eval(st):
    '''字符串打印并执行'''
    _frm = _sys._getframe(1)
    val = eval(st, _frm.f_globals, _frm.f_locals)
    print( f"{st} = {val}")

def print_type(t):
    '''打印类别'''
    _type_name = None
    for tname in dir(_types):
        if tname[0].isupper() and isinstance(t, getattr(_types, tname) ):
            _type_name = tname
            break
    if _type_name is None:
        _type_name = f"{type(t)}"
    print(_type_name)

def print_repr(v):
    '''打印 repr '''
    print(repr(v))

def line_num(depth = 1):
    '''获取当前行号'''
    return _sys._getframe(depth).f_lineno

def func_name(depth = 1):
    '''获取当前函数名字 '''
    return _sys._getframe(depth).f_code.co_name

def func_kws(depth = 1, skip_names = ("self", "cls")):
    ''' 获取函数的参数 '''
    _locals = _sys._getframe(depth).f_locals.copy()
    if skip_names:
        if isinstance(skip_names, str):
            _locals.pop(skip_names, None)
        else:
            for name in skip_names:
                _locals.pop(name, None)
    return _locals



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/3/7 17:34:59

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import numpy as np
from collections import OrderedDict
#===============================================================================
#
#===============================================================================
__all__ = ['load_cld_xyz', 'load_cld_kitti', 'load_label_kitti']

def load_cld_xyz(fname, delimiter=None):
    return np.loadtxt(fname, delimiter=delimiter)

def load_cld_kitti(fname):
    bin_data = np.fromfile(fname, dtype=np.float32)
    return bin_data.reshape((-1, 4))

def load_label_kitti(fname):
    '''返回 kitti 生成器
    :param fname:
    :return:
    '''
    with open(fname) as f:
        for line in f:
            r = line.strip().split()
            obj = OrderedDict()
            obj['type'] = r[0]  # 表明物体类别
            obj['truncation'] = float(r[1]) # 物体是否被截断，从0（非截断）到1（截断）浮动，其中truncated指离开图像边界的对象
            obj['occlusion'] = int(r[2]) # 物体是否被遮挡，整数0，1，2，3表示被遮挡的程度;0：彻底可见  1：小部分遮挡  2：大部分遮挡 3：彻底遮挡
            obj['alpha'] = float(r[3]) # 物体的观察角度，范围：-pi~pi
            obj['bbox'] = (float(r[4]), float(r[5]), float(r[6]), float(r[7]) ) # 物体的2维边界框，左上角和右下角的像素坐标
            obj['dimensions'] = (float(r[8]), float(r[9]), float(r[10])) # 3维物体的尺寸，高、宽、长（单位：米）
            obj['location'] = (float(r[11]), float(r[12]), float(r[13])) # 3维物体的位置 x,y,z（在照相机坐标系下，单位：米）
            obj['yaw'] = float(r[14]) # 3维物体的空间方向：rotation_y，在照相机坐标系下，相对于y轴的旋转角，范围：-pi~pi
            yield obj













#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022/5/23 01:32:08

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''

#===============================================================================
'''     
'''
#===============================================================================
regress_list = [

    ##### 线性回归
    'LinearRegression',
    'Ridge', 'RidgeCV',
    'Lasso', 'LassoCV',
    'LassoLars', 'LassoLarsCV', 'LassoLarsIC', # 最小角度回归
    'ElasticNet', 'ElasticNetCV',
    'SGDRegressor',
    'OrthogonalMatchingPursuit', 'OrthogonalMatchingPursuitCV', #正交匹配追踪算法（OMP）
    'BayesianRidge', 'ARDRegression',
    'TweedieRegressor', 'PoissonRegressor', 'GammaRegressor',
    'RANSACRegressor','HuberRegressor','TheilSenRegressor',
    'QuantileRegressor',

    ##### 核岭回归
    'KernelRidge',

    ##### SVM
    'SVR', 'NuSVR', 'LinearSVR',

    ##### 随机梯度下降
    'SGDRegressor',

    ##### KNN
    'KNeighborsRegressor', 'RadiusNeighborsRegressor',

    ##### 高斯处理
    'GaussianProcessRegressor', 

    ##### 交叉分解
    'PLSRegression', 'PLSCanonical',

    ##### 决策树
    'DecisionTreeRegressor','ExtraTreeRegressor',

    ##### 集成方法
    'RandomForestRegressor', 'ExtraTreesRegressor', 'BaggingRegressor', 
    'GradientBoostingRegressor', 'AdaBoostRegressor', 'VotingRegressor',
    'StackingRegressor', 'HistGradientBoostingRegressor', 

    ##### Isotonic
    'IsotonicRegression', 
    
    ##### 神经网络
    'MLPRegressor', 
]

def get_regress(_name = "Ridge", **kws):
    name = _name.lower()
    if name == 'LinearRegression'.lower():
        from sklearn.linear_model import LinearRegression as Alg
    elif name == 'Ridge'.lower():
        from sklearn.linear_model import Ridge as Alg
    elif name == 'RidgeCV'.lower():
        from sklearn.linear_model import RidgeCV as Alg
    elif name == 'Lasso'.lower():
        from sklearn.linear_model import Lasso as Alg
    elif name == 'LassoCV'.lower():
        from sklearn.linear_model import LassoCV as Alg
    elif name == 'LassoLars'.lower():
        from sklearn.linear_model import LassoLars as Alg
    elif name == 'LassoLarsCV'.lower():
        from sklearn.linear_model import LassoLarsCV as Alg
    elif name == 'LassoLarsIC'.lower():
        from sklearn.linear_model import LassoLarsIC as Alg
    elif name == 'ElasticNet'.lower():
        from sklearn.linear_model import ElasticNet as Alg
    elif name == 'ElasticNetCV'.lower():
        from sklearn.linear_model import ElasticNetCV as Alg

    elif name == 'SGDRegressor'.lower():
        from sklearn.linear_model import SGDRegressor  as Alg

    elif name == 'OrthogonalMatchingPursuit'.lower():
        from sklearn.linear_model import OrthogonalMatchingPursuit as Alg
    elif name == 'OrthogonalMatchingPursuitCV'.lower():
        from sklearn.linear_model import OrthogonalMatchingPursuitCV as Alg

    elif name == 'BayesianRidge'.lower():
        from sklearn.linear_model import BayesianRidge as Alg
    elif name == 'ARDRegression'.lower():
        from sklearn.linear_model import ARDRegression as Alg

    elif name == 'TweedieRegressor'.lower():
        from sklearn.linear_model import TweedieRegressor as Alg
    elif name == 'PoissonRegressor'.lower():
        from sklearn.linear_model import PoissonRegressor as Alg
    elif name == 'GammaRegressor'.lower():
        from sklearn.linear_model import GammaRegressor as Alg

    elif name == 'RANSACRegressor'.lower():
        from sklearn.linear_model import RANSACRegressor  as Alg
    elif name == 'HuberRegressor'.lower():
        from sklearn.linear_model import HuberRegressor  as Alg
    elif name == 'TheilSenRegressor'.lower():
        from sklearn.linear_model import TheilSenRegressor  as Alg
    elif name == 'QuantileRegressor'.lower():
        from sklearn.linear_model import QuantileRegressor  as Alg


    elif name == 'KernelRidge'.lower():
        from sklearn.kernel_ridge import KernelRidge  as Alg

    elif name == 'SVR'.lower():
        from sklearn.svm import SVR  as Alg
    elif name == 'NuSVR'.lower():
        from sklearn.svm import NuSVR  as Alg
    elif name == 'LinearSVR'.lower():
        from sklearn.svm import LinearSVR  as Alg

    elif name == 'KNeighborsRegressor'.lower():
        from sklearn.neighbors import KNeighborsRegressor as Alg
    elif name == 'RadiusNeighborsRegressor'.lower():
        from sklearn.neighbors import RadiusNeighborsRegressor as Alg

    elif name == 'GaussianProcessRegressor'.lower():
        from sklearn.gaussian_process import GaussianProcessRegressor as Alg    

    elif name == 'PLSRegression'.lower():
        from sklearn.cross_decomposition import PLSRegression as Alg    
    elif name == 'PLSCanonical'.lower():
        from sklearn.cross_decomposition import PLSCanonical as Alg    

    elif name == 'DecisionTreeRegressor'.lower():
        from sklearn.tree import DecisionTreeRegressor as Alg    
    elif name == 'ExtraTreeRegressor'.lower():
        from sklearn.tree import ExtraTreeRegressor as Alg    
    
    elif name == 'RandomForestRegressor'.lower():
        from sklearn.ensemble import RandomForestRegressor as Alg
    elif name == 'ExtraTreesRegressor'.lower():
        from sklearn.ensemble import ExtraTreesRegressor as Alg
    elif name == 'BaggingRegressor'.lower():
        from sklearn.ensemble import BaggingRegressor as Alg
    elif name == 'GradientBoostingRegressor'.lower():
        from sklearn.ensemble import GradientBoostingRegressor as Alg
    elif name == 'AdaBoostRegressor'.lower():
        from sklearn.ensemble import AdaBoostRegressor as Alg
    elif name == 'VotingRegressor'.lower():
        from sklearn.ensemble import VotingRegressor as Alg
    elif name == 'StackingRegressor'.lower():
        from sklearn.ensemble import StackingRegressor as Alg
    elif name == 'HistGradientBoostingRegressor'.lower():
        from sklearn.ensemble import HistGradientBoostingRegressor as Alg             

    elif name == 'IsotonicRegression'.lower():
        from sklearn.isotonic import IsotonicRegression as Alg     
        
    elif name == 'MLPRegressor'.lower():
        from sklearn.neural_network import MLPRegressor as Alg

    else:
        raise RuntimeError("Cann't create {}".format(_name))

    return Alg(**kws)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2022.08

@author: laok
@copyright: Apache License, Version 2.0
'''

# ===============================================================================
#                                                                                
# ===============================================================================

import re


class_bin_metric_list = [
    '',


]

def get_class_bin_metric(name):
    pass


class_metric_list = [

]

def get_class_metric_list():
    pass

cluster_metric_list = [


]

def get_cluster_metric(name):
    pass


regress_metric_list = [
    'explained_variance', 'explained_variance_score',
    'max_error',

]

def get_regress_metric(name):
    if name in ['explained_variance', 'explained_variance_score']:
        from sklearn.metrics import explained_variance_score as Alg
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg        
    elif name in [ 'neg_mean_absolute_error', 'mean_absolute_error']:
        from sklearn.metrics import mean_absolute_error as Alg 
    elif name in ['neg_mean_squared_error', 'mean_squared_error']:
        from sklearn.metrics import mean_squared_error as Alg  
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg      
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg  
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg      
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg  
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg      
    elif name == 'max_error':
        from sklearn.metrics import max_error as Alg      
    return Alg





#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2020/6/4 17:25:47

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import inspect, sys, time, traceback, math, re, codecs
from datetime import datetime
#===============================================================================
# 交互式运行
#===============================================================================
_PRINT_FUNC_COUNT = 3

__all__ = ['run']

# 执行计时
def _time_run(_kvfunc, **kws):
    try:
        name, func = _kvfunc
        print( '{}==begin[date:{}]'.format(name, datetime.now().strftime('%Y-%m-%d %H:%M:%S')) )
        t0 = time.time()
        func(**kws)
    except SystemExit:
        exit(0)
    except Exception as e:
        print ( '\nexception in call function---->\n%s' % traceback.format_exc() )
    finally:
        elapse = time.time() - t0
        print ( '{}==end[use time:{}s]\n'.format(name, elapse) )

#从文件提取特定函数名字,主要是用于排序
def _find_order_names(filename, re_pat):
    with codecs.open(filename, encoding='utf8') as f:
        for i,line in enumerate(f, 1):
            res = re_pat.match(line)
            if res:
                yield i,res.group(1)

def run(_opt='last', _suffix ='_lk', **kws):
    '''执行以 _suffix 结尾的函数
    :param _opt: last/first/select/once/all/function_name
    :return:
    '''
    RE_PAT = re.compile(r"^def\s*(.*?{pat})|({pat}.*?)\(.*?\):.*".format(pat = _suffix))

    # 获取所有 _suffix 结尾的函数
    mainMod = sys.modules['__main__']
    kvList = [(k.strip(_suffix), v)
              for k,v in inspect.getmembers(mainMod, inspect.isfunction)
              if k.endswith(_suffix)]

    # 检测是否有测试用例
    if not kvList:
        return

    # 按照代码文本顺序排序
    _kvOrder = {}
    for order, name in  _find_order_names(mainMod.__file__, RE_PAT):
        _kvOrder[name.strip(_suffix)] = order
    kvList.sort(key = lambda pair: _kvOrder[pair[0]])

    # 根据选择,执行函数
    if _opt == 'first':
        return _time_run(kvList[0], **kws)
    elif _opt == 'last':
        return _time_run(kvList[-1], **kws)
    elif _opt == 'all':
        for kv in kvList:
            _time_run(kv, **kws)
    elif _opt in ['select','once', '']:
        listLen = len(kvList)
        while True:
            index_width = math.ceil(math.log10(listLen)) + 2
            name_width = max(len(kv[0]) for kv in kvList) + 2
            fmt = "%0{}s==%-0{}s".format(index_width, name_width)

            for i, kv in enumerate(kvList):
                print(fmt % (f"<{i}>", f"<{kv[0]}>"), end=" ")
                if (i+1) % _PRINT_FUNC_COUNT == 0:
                    print()
            if (i+1) % _PRINT_FUNC_COUNT != 0:
                print()

            i = input('which func do you want to run , -1=quit , -2=run-all:')
            try:
                i = int(i)
            except:
                i = listLen - 1
            if i == -2:
                for kv in kvList:
                    _time_run(kv, **kws)
                return
            elif i == -1:
                return
            else:
                i = (i + listLen) % listLen
                _time_run( kvList[i], **kws)

            if _opt == 'once':
                return
    else :
        if not isinstance(_opt, list):
            run_list = [ _opt ]
        else:
            run_list = _opt

        for _run_op in run_list:
            for kv in kvList:
                if (isinstance(_run_op, str) and kv[0] == _run_op.strip(_suffix) ) \
                        or (inspect.isfunction(_run_op) and kv[1] == _run_op):
                    _time_run(kv, **kws)

